{--------------------------------------------------------------------------
 | Copyright (C) 1992-2002 Dallas Semiconductor MAXIM Corporation.
 | All rights Reserved. Printed in U.S.A.
 | This software is protected by copyright laws of
 | the United States and of foreign countries.
 | This material may also be protected by patent laws of the United States
 | and of foreign countries.
 | This software is furnished under a license agreement and/or a
 | nondisclosure agreement and may only be used or copied in accordance
 | with the terms of those agreements.
 | The mere transfer of this software does not imply any licenses
 | of trade secrets, proprietary technology, copyrights, patents,
 | trademarks, maskwork rights, or any other form of intellectual
 | property whatsoever. Dallas Semiconductor MAXIM retains all ownership rights.
 |--------------------------------------------------------------------------

   Charting for IBVIEW.
}
unit ChartUn;

interface

uses
  {$IFDEF WIN32}
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls;
  {$ELSE}
  Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls;
  {$ENDIF}

const
  { thread messages }
  WM_CloseChart = WM_User + 9;

type
  TChartForm = class(TForm)
    BottomPanel: TPanel;
    BackPanel: TPanel;
    LeftPanel: TPanel;
    CenterPanel: TPanel;
    TitleLabel: TLabel;
    HorizontalTitle: TLabel;
    VerticalTitle: TLabel;
    DrawImage: TImage;
    CheckBoxAutoScale: TCheckBox;
    LabelCurrentReading: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure SizeBars;
    procedure DrawLineChart;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
  public
    { Public declarations }
    MasterWindowHandle : HWND;
    NoPlotIndex : Integer;
    Bars: array[0..64] of TShape;
    BarData: array[0..64] of LongInt;
    HLabels: array[0..4] of TLabel;
    VLabels: array[0..4] of TLabel;
    LineData: array[0..2047] of Real;
    NumLineData : LongInt;
    MaxLineData,MinLineData,LineSpread : Real;
    HighTrip, LowTrip : Real;
    HighBin, LowBin : Integer;
    MaxBarData : LongInt;
    MaxVLabel : LongInt;
    GotChart : Boolean;
    IsBarChart : Boolean;
    procedure BarChart(data: Array of LongInt; NewHLabels: Array of String;
          MainTitle, MainCaption, HTitle, VTitle : String; NewHighBin, NewLowBin: Integer);
    procedure LineChart(data: Array of Real; NumPoints : SmallInt;
                               NewHLabels: Array of String;
                               MainTitle, MainCaption, HTitle, VTitle, tail : String;
                               NewHighTrip, NewLowTrip, MaxDataValue, MinDataValue: Real);
  end;

implementation

{$R *.DFM}

{----------------------------------------------------------------------------}
{ Initialization for this entitiy
}
procedure TChartForm.FormCreate(Sender: TObject);
Var
   i : SmallInt;
begin
   GotChart := False;
   { create the bars, leave invisible }
   For i := 0 to 63 Do
   Begin
      Bars[i] := TShape.Create(Application);
      Bars[i].Parent := CenterPanel;
      Bars[i].Shape := stRectangle;
      Bars[i].Align := alNone;
      Bars[i].Brush.Color := clBlue;
      Bars[i].Brush.Style := bsSolid;
      Bars[i].Pen.Color := clBlue;
      Bars[i].Pen.Mode := pmCopy;
      Bars[i].Pen.Style := psSolid;
      Bars[i].Pen.Width := 1;
      Bars[i].Top := 1;
      Bars[i].Left := 1;
      Bars[i].Height := 10;
      Bars[i].Width := 10;
      Bars[i].Enabled := True;
      Bars[i].Visible := False;
   End;
   { create the labels }
   For i := 0 to 4 Do
   Begin
      HLabels[i] := TLabel.Create(Application);
      HLabels[i].Parent := BottomPanel;
      HLabels[i].AutoSize := True;
      HLabels[i].ParentFont := False;
      HLabels[i].Font.Name := 'Arial';
      HLabels[i].Font.Size := 8;
      HLabels[i].Font.Style := [];
      HLabels[i].Alignment := taCenter;
      HLabels[i].Font.Color := clBlack;
      HLabels[i].Top := 1;
      HLabels[i].Left := 1;
      HLabels[i].Caption := '';
      VLabels[i] := TLabel.Create(Application);
      VLabels[i].Parent := LeftPanel;
      VLabels[i].AutoSize := True;
      VLabels[i].ParentFont := False;
      VLabels[i].Font.Name := 'Arial';
      VLabels[i].Font.Size := 8;
      VLabels[i].Font.Color := clBlack;
      VLabels[i].Font.Style := [];
      VLabels[i].Alignment := taCenter;
      VLabels[i].Color := clBtnFace;
      VLabels[i].Top := 1;
      VLabels[i].Left := 1;
      VLabels[i].Caption := '';
   End;

end;

{----------------------------------------------------------------------------}
{ Create a bar chart
}
procedure TChartForm.BarChart(data: Array of LongInt; NewHLabels: Array of String;
          MainTitle, MainCaption, HTitle, VTitle : String;
          NewHighBin, NewLowBin: Integer);
Var
   i : SmallInt;
Begin
   IsBarChart := True;
   { copy the data into our buffer while looking for max }
   MaxBarData := 1;
   For i := 0 to 63 Do
   Begin
      Bars[i].Visible := False;
      BarData[i] := Data[i];
      If (BarData[i] > MaxBarData) Then
         MaxBarData := BarData[i];
   End;
   { copy the horizontal labels over and set vertical labels }
   For i := 0 to 4 Do
   Begin
      HLabels[i].Caption := '|' + Chr($0D) + NewHLabels[i];
      If (MaxBarData > 10) Then
      Begin
         VLabels[i].Caption := IntToStr(i * (MaxBarData div 4)) + '_';
         MaxVLabel := 4 * (MaxBarData div 4);
      End
      Else
      Begin
         If (i = 4) Then
            VLabels[4].Caption := IntToStr(MaxBarData) + '_'
         Else
            VLabels[i].Caption := '_';
         MaxVLabel := MaxBarData;
      End;
   End;
   LowBin := NewLowBin;
   HighBin := NewHighBin;

   { set titles }
   TitleLabel.Caption := MainTitle;
   HorizontalTitle.Caption := HTitle;
   VerticalTitle.Caption := VTitle;
   Caption := MainCaption;

   { set flag to indicate chart data is here }
   GotChart := True;

   { size the bars }
   SizeBars;

   { loop to make them visible }
   For i := 0 to 63 Do
      Bars[i].Visible := True;
   { loop to show labels }
   For i := 0 to 4 Do
   Begin
      HLabels[i].Visible := True;
      VLabels[i].Visible := True;
   End;
End;

{----------------------------------------------------------------------------}
{ Create a line chart
}
procedure TChartForm.LineChart(data: Array of Real; NumPoints : SmallInt;
                               NewHLabels: Array of String;
                               MainTitle, MainCaption, HTitle, VTitle, tail : String;
                               NewHighTrip, NewLowTrip, MaxDataValue, MinDataValue: Real);
Var
   i : SmallInt;
Begin
   { make the line chart invisible }
   DrawImage.Visible := False;

   IsBarChart := False;
   { copy the data into our buffer while looking for max }
   //MaxLineData := -40.0;
   //MinLineData := 300.0;
   MaxLineData := MinDataValue;
   MinLineData := MaxDataValue;
   If (AnsiPos('Vsens', MainTitle) > 0) Then
   LabelCurrentReading.Caption := Format('%3.1n',[data[99]]) + ' ' + VTitle
   Else LabelCurrentReading.Caption := '';
   { Auto Scale find Min and Max LineData V3.20 }
   If (CheckBoxAutoScale.Checked) Then
   Begin
      For i := 0 to NumPoints-1 Do
      Begin
         LineData[i] := Data[i];
         If (LineData[i] > MaxLineData) Then
            MaxLineData := LineData[i];
         If (LineData[i] < MinLineData) Then
            MinLineData := LineData[i];
      End;
   End
   Else
   Begin
      For i := 0 to NumPoints-1 Do
      Begin
         LineData[i] := Data[i];
      End;
      MaxLineData := MaxDataValue;
      MinLineData := MinDataValue;
   End;
   NumLineData := NumPoints;
   LineSpread := MaxLineData - MinLineData;
   If (LineSpread < 1.0) Then
      LineSpread := MaxLineData;
   If (LineSpread < 0.2) Then
      LineSpread := 0.2;
   { Auto Scale adjustments to Min & Max LineData and LineSpread V3.20 }
   If (CheckBoxAutoScale.Checked) Then
   Begin
      MinLineData := MinLineData - LineSpread / 8;
      MaxLineData := MaxLineData + LineSpread / 8;
      LineSpread := LineSpread + LineSpread * 2 / 8;
   End;
   LowTrip := NewLowTrip;
   HighTrip := NewHighTrip;

   { copy the horizontal labels over and set vertical labels }
   For i := 0 to 4 Do
   Begin
      HLabels[i].Caption := '|' + Chr($0D) + NewHLabels[i];
      If (LineSpread > 1.0) Then
         VLabels[i].Caption := Format('%3.1f',[MinLineData + (i * (LineSpread / 4))]) + tail + '_'
      Else
         VLabels[i].Caption := Format('%3.3f',[MinLineData + (i * (LineSpread / 4))]) + tail + '_';
   End;
   MaxVLabel := 4 * (MaxBarData div 4);

   { set titles }
   TitleLabel.Caption := MainTitle;
   HorizontalTitle.Caption := HTitle;
   VerticalTitle.Caption := VTitle;
   Caption := MainCaption;

   { set flag to indicate chart data is here }
   GotChart := True;

   { size the bars }
   DrawLineChart;

   { make the line chart visible }
   DrawImage.Visible := True;

   { loop to show labels }
   For i := 0 to 4 Do
   Begin
      HLabels[i].Visible := True;
      VLabels[i].Visible := True;
   End;
   If NoPlotIndex > -1 Then NoPlotIndex := NoPlotIndex - 1; { V3.20 } 
End;


{----------------------------------------------------------------------------}
{ Create a bar chart
}
procedure TChartForm.SizeBars;
Var
   i,num : SmallInt;
   bscale : LongInt;
   bwidth,space,hstart : LongInt;
Begin
   CenterPanel.Color := clWhite;
   { calculate a scale factor }
   space := CenterPanel.ClientWidth div 150;
   If (space = 0) Then
      space := 1;
   bscale := (CenterPanel.ClientHeight div 8) * 7;
   bwidth := (CenterPanel.ClientWidth div 65) - space;
   { loop and scale each bar }
   For i := 0 to 63 Do
   Begin
      { set the color of the bar }
      If (i < LowBin) Then
         Bars[i].Brush.Color := clBlue
      Else If (i > HighBin) Then
         Bars[i].Brush.Color := clRed
      Else If (i >= LowBin) And (i <= HighBin) Then
         Bars[i].Brush.Color := clBlack
      Else
         Bars[i].Brush.Color := clGreen;
      Bars[i].Pen.Color := Bars[i].Brush.Color;
      Bars[i].Height := Trunc((BarData[i] / MaxBarData) * bscale);
      Bars[i].Width := bwidth;
      Bars[i].Top := CenterPanel.ClientHeight - Bars[i].Height;
      Bars[i].Left := i * (bwidth + space);
   End;
   { position each of the labels }
   hstart := BottomPanel.ClientWidth - CenterPanel.ClientWidth;
   bscale := Trunc((MaxVLabel / MaxBarData) * bscale);
   For i := 0 to 4 Do
   Begin
      If (i = 0) Then
         num := 0
      Else
         num := 16 * i - 1;
      HLabels[i].Left := hstart + (num * (bwidth + space)) + bwidth div 2 -
                         HLabels[i].Width div 2;
      HLabels[i].Top := 0;
      VLabels[i].Top :=  LeftPanel.ClientHeight -
                         i * (bscale div 4) -
                         VLabels[i].Height;
      VLabels[i].Left := LeftPanel.ClientWidth - VLabels[i].Width;
   End;
   { center horizontal title }
   HorizontalTitle.Left := (HLabels[4].Left + HLabels[4].Width div 2 - hstart)
                            div 2 + hstart - HorizontalTitle.Width div 2;
   HorizontalTitle.Top := BottomPanel.ClientHeight - HorizontalTitle.Height;
End;

{----------------------------------------------------------------------------}
{ Resize chart
}
procedure TChartForm.FormResize(Sender: TObject);
begin
   If (GotChart) Then
   Begin
      If (IsBarChart) Then
         SizeBars
      Else
         DrawLineChart;

      { Resize label }
       LabelCurrentReading.Left := TitleLabel.Left + (TitleLabel.Width * 13 div 16);
   End;
end;

{----------------------------------------------------------------------------}
{ Create a line chart
}
procedure TChartForm.DrawLineChart;
Var
   i,x,y,hstart,blocksize : LongInt;
   bscale : LongInt;
   NewColor, LastColor : TColor;
Begin
   CenterPanel.Color := clBtnFace;
   { make the bmp for the char max size }
   DrawImage.Width := Screen.Width;
   DrawImage.Height := Screen.Height;
   DrawImage.Canvas.FillRect(DrawImage.ClientRect);

   { clear the previous image }
   DrawImage.Canvas.Brush.Style := bsSolid;
   DrawImage.Canvas.Brush.Color := clWhite;
   DrawImage.Width := CenterPanel.ClientWidth - HLabels[4].Width div 2;
   DrawImage.Height := CenterPanel.ClientHeight - TitleLabel.Height - VLabels[4].Height;
   DrawImage.Left := 0;
   DrawImage.Top := TitleLabel.Height + VLabels[4].Height;
   DrawImage.Canvas.FillRect(DrawImage.ClientRect);
   bscale := DrawImage.ClientHeight;
   blocksize := DrawImage.Width div 300;
   If (blocksize < ((DrawImage.Width div (NumLineData - 1)) div 2)) Then
      blocksize := DrawImage.Width div 200;
   If (blocksize < 1) Then
      blocksize := 3;

   { loop to draw each of the samples }
   LastColor := clBlack;
   For i := NoPlotIndex + 1 to NumLineData - 1 Do
   Begin
      { figure out what color to use }
      If (LineData[i] < LowTrip) Then
         NewColor := clBlue
      Else If (LineData[i] > HighTrip) Then
         NewColor := clRed
      Else If (LineData[i] >= LowTrip) And (LineData[i] <= HighTrip) Then
         NewColor := clBlack
      Else
         NewColor := clGreen;
      If (LastColor = NewColor) Then
         DrawImage.Canvas.Pen.Color := NewColor
      Else
         DrawImage.Canvas.Pen.Color := clBlack;
      { draw the line }
      y := Trunc(DrawImage.Height - (((LineData[i] - MinLineData) / LineSpread) * bscale));
      x := Trunc((DrawImage.Width / (NumLineData - 1)) * i);
      If (i = 0) or (i = NoPlotIndex + 1) Then
         DrawImage.Canvas.MoveTo(X, Y);
      DrawImage.Canvas.LineTo(X, Y);
      DrawImage.Canvas.Brush.Color := NewColor;
      DrawImage.Canvas.Pen.Color := NewColor;
      LastColor := NewColor;
      DrawImage.Canvas.Rectangle(X - blocksize, Y - blocksize, X + blocksize, Y + blocksize);
   End;

   { position each of the labels }
   hstart := BottomPanel.ClientWidth - CenterPanel.ClientWidth;
   For i := 0 to 4 Do
   Begin
      HLabels[i].Left := hstart +  i * (DrawImage.Width div 4) - HLabels[i].Width div 2;
      HLabels[i].Top := 0;
      VLabels[i].Top :=  LeftPanel.ClientHeight - i * (DrawImage.Height div 4) -
                         VLabels[i].Height;
      VLabels[i].Left := LeftPanel.ClientWidth - VLabels[i].Width;
   End;
   { center horizontal title }
   HorizontalTitle.Left := hstart +  DrawImage.Width div 2 - HorizontalTitle.Width div 2;
   HorizontalTitle.Top := BottomPanel.ClientHeight - HorizontalTitle.Height;
End;

{----------------------------------------------------------------------------}
{ Catch the OnClose Form event   ( V3.20 )
}
procedure TChartForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
   { Send message to Form that created this one (i.e., DS2450 or DS2438 viewer) }
   PostMessage(MasterWindowHandle,WM_CloseChart,WindowHandle,0);
end;

end.
